@@ -243,6 +243,24 @@ public interface Session {
      * @return
      */
     int getMailLimit();
+    
+    /**
+     * Sets a limit for the number of objects that will be reported when syncing a folder. If on the server more objects than the specified
+     * limit are stored in a folder, only the newest ones (upto the limit) will be used for synchronization, older one will be treated as
+     * not present. A value <= 0 specifies that no limit should be used and all objects should be synchronized.
+     * 
+     * @param limit
+     */
+    void setObjectsLimit(int limit);
+
+    /**
+     * Retrieves the currently set limit for objects in a folder.
+     * 
+     * @return
+     */
+    int getObjectsLimit();
+    
+   
 
     // -- Listening / waiting for changes on the server --
     /**
@@ -871,13 +889,6 @@ public interface Session {
      */
     void setFolderIdLengthLimit(int limit);
 
-    /**
-     * Sets the current client ip address.
-     * 
-     * @param ip
-     */
-    void setClientIp(String ip);
-
     /**
      * Retrieves the current saved client ip address.
      * 
@@ -886,11 +897,11 @@ public interface Session {
     String getClientIp();
 
     /**
-     * Resets the current client ip address in the local and server session.
+     * Updates the current client ip address in the local and server session.
      * 
      * @param ip
      */
-    void resetClientIp(String ip);
+    void updateClientIp(String ip);
 
     Map<String, String> getXRequestHeaders();
 
