@@ -108,5 +108,5 @@ public interface SessionManager {
 	Session getSession(String user, String password, String protocol, String device, SessionInitializer initializer,
 			String clientIP, Map<String, String> xHeaders) throws USMException;
 
-	boolean exitsSessionInDB(String user, String protocol, String device) throws USMException;
+	boolean existsSessionInDB(String user, String protocol, String device) throws USMException;
 }
