@@ -60,6 +60,7 @@ import org.json.JSONException;
 import org.json.JSONObject;
 import com.openexchange.usm.api.contenttypes.ContentType;
 import com.openexchange.usm.api.contenttypes.DefaultContentTypes;
+import com.openexchange.usm.api.contenttypes.OXFolderContent;
 import com.openexchange.usm.api.contenttypes.FolderContentType;
 import com.openexchange.usm.api.contenttypes.FolderContentTypeTransferHandler;
 import com.openexchange.usm.api.contenttypes.UnsupportedContentOperationException;
@@ -86,6 +87,10 @@ import com.openexchange.usm.ox_json.OXJSONAccess;
  */
 public class FolderContentTypeTransferHandlerImpl extends AbstractTransferHandler implements FolderContentTypeTransferHandler {
 
+    private static final String ONE = "1";
+
+    private static final String IGNORE_TRANSLATION = "ignoreTranslation";
+
     private static final int FOLDER_ID_LENGTH_LIMIT = 255;
 
     private static final String ALLOWED_MODULES = "allowed_modules";
@@ -99,6 +104,16 @@ public class FolderContentTypeTransferHandlerImpl extends AbstractTransferHandle
         readFolder(checkContentType(object), requestedFields);
     }
 
+    @Override
+    public Folder[] readFolderContent(Folder folder) throws USMException {
+        return readFolderContent(folder, folder.getSession().getFieldFilter(folder.getContentType()));
+    }
+
+    @Override
+    public OXFolderContent readFolderContent(Folder folder, int limit) throws USMException {
+        return new OXFolderContent(true, readFolderContent(folder, folder.getSession().getFieldFilter(folder.getContentType())));
+    }
+    
     @Override
     public Folder[] readFolderContent(Folder folder, BitSet requestedFields) throws USMException {
         Session session = folder.getSession();
@@ -109,9 +124,7 @@ public class FolderContentTypeTransferHandlerImpl extends AbstractTransferHandle
         String errorOnDuplicateNamePersistentField = session.getPersistentField(UtilConstants.ERROR_ON_DUPLICATE_NAME_DB_FIELD);
         if (errorOnDuplicateNamePersistentField != null && errorOnDuplicateNamePersistentField.length() > 0)
             parameters.put(UtilConstants.ERROR_ON_DUPLICATE_NAME, errorOnDuplicateNamePersistentField);
-        if(folder.getSession().getObjectsLimit() > 0) {
-            addObjectsLimitParameters(folder.getSession().getObjectsLimit(), parameters);
-        }
+        addObjectsLimitParameters(session, parameters);
         DataObject[] listOfResults = performActionAndCheckRights(UtilConstants.ACTION_LIST, folder, requestedFields, parameters);
         Folder[] result = tranformToFolders(listOfResults);
         for (Folder f : result) {
@@ -270,6 +283,7 @@ public class FolderContentTypeTransferHandlerImpl extends AbstractTransferHandle
         Map<String, String> parameters = new HashMap<String, String>();
         addTreeParameter(session, parameters);
         parameters.put(UtilConstants.COLUMNS, createColumnsParameter(requestedFields));
+        parameters.put(IGNORE_TRANSLATION, ONE);
         return parameters;
     }
 
