@@ -52,7 +52,6 @@ package com.openexchange.usm.contenttypes.mail.impl;
 import java.util.ArrayList;
 import java.util.BitSet;
 import java.util.Collections;
-import java.util.Comparator;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
@@ -63,6 +62,7 @@ import org.json.JSONObject;
 import com.openexchange.usm.api.contenttypes.ContentType;
 import com.openexchange.usm.api.contenttypes.ContentTypeField;
 import com.openexchange.usm.api.contenttypes.DefaultContentTypes;
+import com.openexchange.usm.api.contenttypes.OXFolderContent;
 import com.openexchange.usm.api.contenttypes.MailContentType;
 import com.openexchange.usm.api.contenttypes.UnsupportedContentOperationException;
 import com.openexchange.usm.api.exceptions.OXCommunicationException;
@@ -83,6 +83,8 @@ import com.openexchange.usm.ox_json.OXResource;
 
 public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler {
 
+    private static final String _RECEIVED_DATE_COLUMN = "610";
+
     private static final String DROP_PREFIX = "dropPrefix";
 
     private static final String COM_OPENEXCHANGE_MAIL_CONVERSION_SEQUENCEID = "com.openexchange.mail.conversion.sequenceid";
@@ -389,18 +391,26 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
         return UtilConstants.MAIL_PATH;
     }
 
+    @Override
+    public OXFolderContent readFolderContent(Folder folder, int limit) throws USMException {
+        return new OXFolderContent(true, readFolderContent(folder, folder.getSession().getFieldFilter(folder.getElementsContentType()), limit));
+    }
+
     @Override
     public DataObject[] readFolderContent(Folder folder, BitSet requestedFields) throws USMException {
+        return readFolderContent(folder, requestedFields, Integer.MAX_VALUE);
+    }
+
+    private DataObject[] readFolderContent(Folder folder, BitSet requestedFields, int limit) throws USMException {
         if (AbstractTransferHandler.CHECK_ACCESS_RIGHTS_BEFORE_FOLDER_READ && !hasFolderReadRights(folder))
             return SyncResult.EMPTY_DATA_OBJECT_ARRAY;
         Session session = folder.getSession();
         long startDate = session.getStartDate();
         long endDate = session.getEndDate();
-        int mailLimit = session.getMailLimit();
         try {
-            if (startDate == 0L && endDate == Long.MAX_VALUE && mailLimit <= 0)
-                return super.readFolderContent(folder, requestedFields);
-            DataObject[] idList = super.readFolderContent(folder, _getAllBitSet);
+            if (startDate == 0L && endDate == Long.MAX_VALUE)
+                return super.readCompleteFolderContent(folder, requestedFields);
+            DataObject[] idList = super.readCompleteFolderContent(folder, _getAllBitSet);
             List<DataObject> filteredList = new ArrayList<DataObject>();
             for (DataObject o : idList) {
                 Long t = (Long) o.getFieldContent(_receivedDateIndex);
@@ -409,23 +419,9 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
             }
             if (filteredList.isEmpty())
                 return SyncResult.EMPTY_DATA_OBJECT_ARRAY;
-            if (mailLimit > 0 && filteredList.size() > mailLimit) {
-                Collections.sort(filteredList, new Comparator<DataObject>() {
-
-                    public int compare(DataObject a, DataObject b) {
-                        // Sort descending, if no time is present, treat as newest
-                        Long ta = (Long) a.getFieldContent(_receivedDateIndex);
-                        Long tb = (Long) b.getFieldContent(_receivedDateIndex);
-                        if (ta == null) {
-                            return (tb == null) ? 0 : -1;
-                        } else if (tb == null) {
-                            return 1;
-                        } else {
-                            return tb.compareTo(ta);
-                        }
-                    }
-                });
-                filteredList.subList(mailLimit, filteredList.size()).clear();
+            if (filteredList.size() > limit) {
+                // We could add a check here to limit the number of emails read, but we would have to extract the latest sync state, which
+                // also costs some time. For now we do nothing special.
             }
             JSONArray ids = new JSONArray();
             for (DataObject o : filteredList) {
@@ -641,4 +637,13 @@ public class MailContentTypeTransferHandlerImpl extends AbstractTransferHandler
         }
         return conversionObject;
     }
+
+    @Override
+    protected void addObjectsLimitParameters(Session session, Map<String, String> parameters) {
+        int mailLimit = session.getMailLimit();
+        int objectLimit = session.getObjectsLimit();
+        int limit = (mailLimit <= 0 || (objectLimit > 0 && objectLimit < mailLimit)) ? objectLimit : mailLimit;
+        if (limit > 0)
+            addObjectsLimitParameters(limit, _RECEIVED_DATE_COLUMN, parameters);
+    }
 }
