@@ -49,14 +49,14 @@
 package com.openexchange.usm.connector.commands;
 
 import static com.openexchange.usm.connector.commands.CommandConstants.*;
-
+import java.util.ArrayList;
+import java.util.List;
+import java.util.UUID;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpSession;
-
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
-
 import com.openexchange.usm.api.contenttypes.DefaultContentTypes;
 import com.openexchange.usm.api.database.DatabaseAccessException;
 import com.openexchange.usm.api.exceptions.USMException;
@@ -70,23 +70,26 @@ import com.openexchange.usm.json.USMJSONAPIException;
 import com.openexchange.usm.json.USMJSONServlet;
 import com.openexchange.usm.json.response.ResponseObject;
 import com.openexchange.usm.json.response.ResponseStatusCode;
+import com.openexchange.usm.session.dataobject.DataObjectSet;
 import com.openexchange.usm.util.JSONToolkit;
 
 /**
  * Handler for the USM-Ping Command. 
  * @author ldo
- *
  */
 public class PingHandler extends NormalCommandHandler {
 
+    private static final String _DEFAULT_INBOX_FOLDER_KEY = "json.DefaultInboxFolder";
+
 	private static final String INTERVAL = "interval";
 	private static final String[] REQUIRED_PARAMETERS = { SESSIONID };
 	private static final String[] OPTIONAL_PARAMETERS = { OBJECTIDS };
 
-	private Folder _defaultInbox;
+    private DataObjectSet _allCachedFolders;
 
-	public PingHandler(USMJSONServlet servlet, HttpSession httpSession, JSONObject parameters,
-			HttpServletRequest request) throws USMJSONAPIException {
+    private boolean _watchFolderHierarchyChanges = true;
+
+    public PingHandler(USMJSONServlet servlet, HttpSession httpSession, JSONObject parameters, HttpServletRequest request) throws USMJSONAPIException {
 		super(servlet, httpSession, parameters, request);
 		_servlet.newPingRequestArrived();
 	}
@@ -95,29 +98,35 @@ public class PingHandler extends NormalCommandHandler {
 	public ResponseObject handleRequest() throws USMJSONAPIException {
 
 		JSONArray objectUUIDs = _parameters.has(OBJECTIDS) ? getJSONArray(_parameters, OBJECTIDS) : null;
-		boolean watchFolderHierarchyChanges = true;
-		if (objectUUIDs != null) {
-			watchFolderHierarchyChanges = isContextUUIDRequested(objectUUIDs);
-		}
 		try {
-			ObjectChanges changes = getSession().waitForChanges(watchFolderHierarchyChanges,
-					toOXIdsStringArray(objectUUIDs), 0);
+            String[] oxIDs = extractOXIdsFromUUIDArray(objectUUIDs);
+            ObjectChanges changes = getSession().waitForChanges(_watchFolderHierarchyChanges, oxIDs, 0);
 			JSONArray resultArray = null;
 			if (changes != null) {
 				resultArray = toUUIDsJSONArray(changes.getContentChangeParentIDs());
 				if (changes.hasFolderStructureChanged())
 					resultArray.put(getSession().getContextUUID().toString());
 			}
-			if (!hasDefaultInboxChanges(changes) && getDefaultInbox() != null) {
+            String inboxId = getDefaultInboxId();
+            if (inboxId != null && !hasDefaultInboxChanges(changes, inboxId)) {
 				if (resultArray == null)
 					resultArray = new JSONArray();
-				long newestTimestamp = getSession().getNewestTimestamp(getDefaultInbox().getID());
-				if (newestTimestamp > 0
-						&& System.currentTimeMillis() - newestTimestamp >= _servlet.getInboxPollingInterval()) {
-					SyncResult syncResult = getSession().syncChangesWithServer(getDefaultInbox().getID(),
-							newestTimestamp, Session.NO_LIMIT, null, false, null);
-					if (syncResult.getChanges().length > 0 && getDefaultInbox().getUUID() != null)
-						resultArray.put(getDefaultInbox().getUUID());
+                long newestTimestamp = getSession().getNewestTimestamp(inboxId);
+                if (newestTimestamp > 0 && System.currentTimeMillis() - newestTimestamp >= _servlet.getInboxPollingInterval()) {
+                    // clear cached value so that it will be updated from OX configuration
+                    getSession().setCustomProperty(_DEFAULT_INBOX_FOLDER_KEY, null);
+                    Folder defaultInbox = (Folder) getAllCachedFolders().get(inboxId);
+                    if (defaultInbox != null) {
+                        SyncResult syncResult = getSession().syncChangesWithServer(
+                            inboxId,
+                            newestTimestamp,
+                            Session.NO_LIMIT,
+                            null,
+                            false,
+                            null);
+                        if (syncResult.getChanges().length > 0 && defaultInbox.getUUID() != null)
+                            resultArray.put(defaultInbox.getUUID());
+                    }
 				}
 			}
 			JSONObject data = new JSONObject();
@@ -135,92 +144,91 @@ public class PingHandler extends NormalCommandHandler {
 		}
 	}
 
-	private Folder getDefaultInbox() throws USMException {
-		if (_defaultInbox != null)
-			return _defaultInbox;
-		String defaultInboxID = JSONToolkit.getString(
-				getSession().getOXUserConfiguration("modules", "mail", "defaultFolder"), "inbox");
-		if (defaultInboxID == null)
-			return _defaultInbox;
-		_defaultInbox = getSession().getCachedFolder(defaultInboxID);
-		return _defaultInbox;
+    private DataObjectSet getAllCachedFolders() throws DatabaseAccessException, USMSQLException {
+        if (_allCachedFolders == null) {
+            _allCachedFolders = new DataObjectSet(getSession().getCachedFolders());
+        }
+        return _allCachedFolders;
+    }
+
+    private String getDefaultInboxId() throws USMException {
+        Object o = getSession().getCustomProperty(_DEFAULT_INBOX_FOLDER_KEY);
+        if (o instanceof String)
+            return (String) o;
+        String defaultInboxID = JSONToolkit.getString(getSession().getOXUserConfiguration("modules", "mail", "defaultFolder"), "inbox");
+        getSession().setCustomProperty(_DEFAULT_INBOX_FOLDER_KEY, defaultInboxID);
+        return defaultInboxID;
 	}
 
-	private boolean hasDefaultInboxChanges(ObjectChanges changes) throws USMException {
-		if (getDefaultInbox() == null || changes == null)
+    private static boolean hasDefaultInboxChanges(ObjectChanges changes, String inboxId) {
+        if (changes == null)
 			return false;
 
 		for (String folderID : changes.getContentChangeParentIDs()) {
-			if (folderID.equals(getDefaultInbox().getID()))
+            if (folderID.equals(inboxId))
 				return true;
 		}
 		return false;
 	}
 
-	private boolean isContextUUIDRequested(JSONArray objectUUIDs) throws USMJSONAPIException {
-		int length = objectUUIDs.length();
-		for (int i = 0; i < length; i++) {
-			String uuid = getString(objectUUIDs, i);
-			if (isContextUUID(uuid))
-				return true;
-		}
-		return false;
-	}
-
-	private boolean isContextUUID(String uuid) throws USMJSONAPIException {
+    private UUID getContextUUID() {
 		try {
-			if (getSession().getContextUUID().equals(extractUUIDFromString(uuid)))
-				return true;
+            return getSession().getContextUUID();
 		} catch (DatabaseAccessException e) {
 			USMJSONAPIException.createInternalError(ConnectorBundleErrorCodes.PING_NO_DB_ACCESS, e);
 		} catch (USMSQLException e) {
 			USMJSONAPIException.createInternalError(ConnectorBundleErrorCodes.PING_SQL_ERROR, e);
 		}
-		return false;
+        return null; // not reached
 	}
 
-	private String[] toOXIdsStringArray(JSONArray jsonArray) throws USMJSONAPIException {
+    private String[] extractOXIdsFromUUIDArray(JSONArray jsonArray) throws USMJSONAPIException, DatabaseAccessException, USMSQLException {
 		if (jsonArray == null)
 			return null;
 		int length = jsonArray.length();
-		String[] stringArray = new String[length - (isContextUUIDRequested(jsonArray) ? 1 : 0)];
-		int j = 0;
+        List<String> oxIDs = new ArrayList<String>(length);
+        UUID contextUUID = getContextUUID();
 		for (int i = 0; i < length; i++) {
-			String uuid = getString(jsonArray, i);
-			if (!isContextUUID(uuid)) {
-				Folder folder = getFolderByUUID(uuid);
-				stringArray[j++] = folder.getID();
+            String id = getString(jsonArray, i);
+            if (isSpecialFolderType(id)) {
+                Folder f = (Folder) getAllCachedFolders().get(id);
+                if (f != null)
+                    oxIDs.add(f.getID());
+            } else {
+                UUID uuid = extractUUIDFromString(id);
+                if (uuid.equals(contextUUID)) {
+                    _watchFolderHierarchyChanges = true;
+                } else {
+                    Folder f = (Folder) getAllCachedFolders().get(uuid);
+                    if (f != null)
+                        oxIDs.add(f.getID());
+                }
 			}
 		}
-		return stringArray;
+        return oxIDs.toArray(new String[oxIDs.size()]);
 	}
 
-    private JSONArray toUUIDsJSONArray(String[] idsArray) throws DatabaseAccessException, USMSQLException, JSONException {
+    private JSONArray toUUIDsJSONArray(String[] idsArray) throws DatabaseAccessException, USMSQLException {
+        DataObjectSet allFolders = getAllCachedFolders();
         JSONArray result = new JSONArray();
         for (int i = 0; i < idsArray.length; i++) {
-            if (isSpecialFolderType(idsArray[i]))
+            if (isSpecialFolderType(idsArray[i])) {
                 result.put(idsArray[i]);
-            else {
-                Folder cachedFolder = getSession().getCachedFolder(idsArray[i]);
-                if (cachedFolder != null && isOfDefaultContentType(cachedFolder.getElementsContentTypeID()) && cachedFolder.getUUID() != null)
+            } else {
+                Folder cachedFolder = (Folder) allFolders.get(idsArray[i]);
+                if (cachedFolder != null && cachedFolder.getUUID() != null && isOfDefaultContentType(cachedFolder.getElementsContentTypeID()))
                     result.put(cachedFolder.getUUID().toString());
             }
         }
         return result;
     }
 	
-    private boolean isSpecialFolderType(String id) {
+    private static boolean isSpecialFolderType(String id) {
         return DefaultContentTypes.GROUPS_ID.equals(id) || DefaultContentTypes.RESOURCES_ID.equals(id);
     }
 
-	private boolean isOfDefaultContentType(String contentTypeID) {
-		return DefaultContentTypes.FOLDER_ID.equals(contentTypeID)
-				|| DefaultContentTypes.CONTACTS_ID.equals(contentTypeID)
-				|| DefaultContentTypes.CALENDAR_ID.equals(contentTypeID)
-				|| DefaultContentTypes.TASK_ID.equals(contentTypeID)
-				|| DefaultContentTypes.MAIL_ID.equals(contentTypeID)
-				|| DefaultContentTypes.GROUPS_ID.equals(contentTypeID)
-				|| DefaultContentTypes.RESOURCES_ID.equals(contentTypeID);
+    private static boolean isOfDefaultContentType(String contentTypeID) {
+        return DefaultContentTypes.FOLDER_ID.equals(contentTypeID) || DefaultContentTypes.CONTACTS_ID.equals(contentTypeID) || DefaultContentTypes.CALENDAR_ID.equals(contentTypeID) || DefaultContentTypes.TASK_ID.equals(contentTypeID) || DefaultContentTypes.MAIL_ID.equals(contentTypeID);
 	}
 
 	@Override
