@@ -94,7 +94,7 @@ public class ResolveIDsHandler extends BaseCommandHandler {
 			String password = getStringParameter(PASSWORD);
 			String device = getStringParameter(DEVICE);
 
-			boolean sessionAlreadyInDB = _servlet.getSessionManager().exitsSessionInDB(user, ServletConstants.PROTOCOL,
+			boolean sessionAlreadyInDB = _servlet.getSessionManager().existsSessionInDB(user, ServletConstants.PROTOCOL,
 					device);
 
 			if (!sessionAlreadyInDB)
