@@ -79,10 +79,12 @@ public class EmailSizeFilter implements DataObjectFilter {
 
 	@Override
     public boolean accept(DataObject object) {
-	    boolean isTooBig = getMailSize(object) > _maxEmailSize;
-	    _hadTooBigEmails |= isTooBig;
-	    return !isTooBig;
-	}
+	    if(_maxEmailSize <= 0)
+	        return true;
+        boolean isTooBig = getMailSize(object) > _maxEmailSize;
+        _hadTooBigEmails |= isTooBig;
+        return !isTooBig;
+    }
 
 	@Override
     public boolean addOperationToResult(ChangeState action, DataObject object) {
