@@ -53,11 +53,11 @@ import static com.openexchange.usm.json.ServletConstants.*;
 import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.InputStreamReader;
+import java.io.OutputStreamWriter;
 import java.io.UnsupportedEncodingException;
 import java.io.Writer;
 import java.util.Map;
 import javax.servlet.ServletException;
-import javax.servlet.ServletOutputStream;
 import javax.servlet.http.HttpServlet;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
@@ -85,8 +85,6 @@ import com.openexchange.usm.util.AccessLimiter;
 import com.openexchange.usm.util.Toolkit;
 
 public class USMJSONServlet extends HttpServlet {
-    
-    private static final boolean _USE_RESPONSE_STREAMING = false;
 
     private static final int ONE_SECOND = 1000;
 
@@ -136,7 +134,7 @@ public class USMJSONServlet extends HttpServlet {
     private transient int _timefilterLimitFuture;
 
     private transient int _inboxPollingInterval;
-    
+
     private transient int _maxEmailSize;
 
     private transient AccessLimiter _accessLimiter;
@@ -343,22 +341,12 @@ public class USMJSONServlet extends HttpServlet {
                 break;
             }
             response.setContentType(ServletConstants.USM_JSON_CONTENT_TYPE);
-            if(_USE_RESPONSE_STREAMING) {
-                Writer w = response.getWriter();
-                if(w != null) {
-                    result.write(w);
-                    w.flush();
-                    w.close();
-                }
-            } else {
-                byte[] data = result.toString().getBytes(ServletConstants.UTF_ENCODING_8);
-                response.setContentLength(data.length);
-                ServletOutputStream os = response.getOutputStream();
-                if (os != null) {
-                    os.write(data, 0, data.length);
-                    os.flush();
-                    os.close();
-                }
+            response.setContentLength(computeContentLength(result));
+            Writer w = response.getWriter();
+            if (w != null) {
+                result.write(w);
+                w.flush();
+                w.close();
             }
             if (_journal.isDebugEnabled()) {
                 _journal.debug(generateTimeLog(handler, command, System.currentTimeMillis() - startTime));
@@ -369,6 +357,14 @@ public class USMJSONServlet extends HttpServlet {
         }
     }
 
+    private static int computeContentLength(ResponseObject result) throws JSONException, IOException {
+        ByteCountOutputStream counter = new ByteCountOutputStream();
+        OutputStreamWriter outputStreamWriter = new OutputStreamWriter(counter, ServletConstants.UTF_ENCODING_8);
+        result.write(outputStreamWriter);
+        outputStreamWriter.close();
+        return counter.getByteCount();
+    }
+
     private void sendError(HttpServletResponse response, int status) throws IOException {
         if (!response.isCommitted()) {
             try {
