@@ -48,21 +48,41 @@
  */
 package com.openexchange.usm.mimemail;
 
-import java.io.*;
+import java.io.ByteArrayInputStream;
+import java.io.ByteArrayOutputStream;
+import java.io.IOException;
+import java.io.UnsupportedEncodingException;
 import java.text.SimpleDateFormat;
-import java.util.*;
-
+import java.util.Date;
+import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.Locale;
+import java.util.Map;
+import java.util.Properties;
+import java.util.Set;
 import javax.activation.DataHandler;
-import javax.mail.*;
-import javax.mail.internet.*;
+import javax.mail.Header;
+import javax.mail.Message;
+import javax.mail.MessagingException;
+import javax.mail.Part;
+import javax.mail.Session;
+import javax.mail.internet.AddressException;
+import javax.mail.internet.InternetHeaders;
+import javax.mail.internet.MimeBodyPart;
+import javax.mail.internet.MimeMessage;
+import javax.mail.internet.MimeMultipart;
+import javax.mail.internet.MimePart;
+import javax.mail.internet.MimeUtility;
 import javax.mail.util.ByteArrayDataSource;
-
-import org.json.*;
-
+import org.json.JSONArray;
+import org.json.JSONException;
+import org.json.JSONObject;
 import com.openexchange.exception.OXException;
-import com.openexchange.mail.mime.*;
 import com.openexchange.mail.mime.ContentDisposition;
 import com.openexchange.mail.mime.ContentType;
+import com.openexchange.mail.mime.QuotedInternetAddress;
 import com.openexchange.usm.json.ConnectorBundleErrorCodes;
 import com.openexchange.usm.json.USMJSONAPIException;
 import com.openexchange.usm.json.response.ResponseStatusCode;
@@ -478,7 +498,7 @@ public class MimeMailBuilder {
 
 	}
 
-	static List<String> specialHeaderNamesList = new ArrayList<String>();
+	static Set<String> specialHeaderNamesList = new HashSet<String>();
 	static {
 		specialHeaderNamesList.add(CONTENT_TYPE);
 		specialHeaderNamesList.add(CONTENT_DISPOSITION);
@@ -489,13 +509,10 @@ public class MimeMailBuilder {
 		specialHeaderNamesList.add(MIME_VERSION);
 		specialHeaderNamesList.add(X_MAILER);
 	}
-	static List<String> knownAddressHeaderNamesList = new ArrayList<String>();
 
+	static Set<String> knownAddressHeaderNamesList = new HashSet<String>();
 	static {
 		knownAddressHeaderNamesList.add(FROM_ADDRESSLIST_HEADER);
-		knownAddressHeaderNamesList.add(TO_ADDRESSLIST_HEADER);
-		knownAddressHeaderNamesList.add(CC_ADDRESSLIST_HEADER);
-		knownAddressHeaderNamesList.add(BCC_ADDRESSLIST_HEADER);
 		knownAddressHeaderNamesList.add(REPLY_TO_ADDRESSLIST_HEADER);
 		knownAddressHeaderNamesList.add(SENDER_ADDRESSLIST_HEADER);
 		knownAddressHeaderNamesList.add(ERRORS_TO_ADDRESSLIST_HEADER);
@@ -507,6 +524,13 @@ public class MimeMailBuilder {
 		knownAddressHeaderNamesList.add(DISPOSITION_NOTIFICATION_TO_ADDRESSLIST_HEADER);
 	}
 
+    static Map<String, Message.RecipientType> _RECIPIENT_ADDRESS_HEADERS = new HashMap<String, Message.RecipientType>();
+    static {
+        _RECIPIENT_ADDRESS_HEADERS.put(TO_ADDRESSLIST_HEADER, Message.RecipientType.TO);
+        _RECIPIENT_ADDRESS_HEADERS.put(CC_ADDRESSLIST_HEADER, Message.RecipientType.CC);
+        _RECIPIENT_ADDRESS_HEADERS.put(BCC_ADDRESSLIST_HEADER, Message.RecipientType.BCC);
+    }
+
 	private Enumeration<Header> parseXOriginalHeaders(String xOriginalHeaders) throws USMJSONAPIException {
 		ByteArrayInputStream is = new ByteArrayInputStream(Toolkit.decodeBase64(xOriginalHeaders));
 		try {
@@ -578,7 +602,10 @@ public class MimeMailBuilder {
 			if (headers != null) {
 				Iterable<String> headerNames = JSONToolkit.keys(headers);
 				for (String headerName : headerNames) {
-					if (knownAddressHeaderNamesList.contains(headerName)) {
+				    Message.RecipientType recipientType = _RECIPIENT_ADDRESS_HEADERS.get(headerName);
+				    if(recipientType != null) {
+				        appendRecipientHeader(part, headers, recipientType, headerName);
+				    } else if (knownAddressHeaderNamesList.contains(headerName)) {
 						setHeaderConditional(part, headerName, getAddress(headers, headerName, null));
 					} else if (!specialHeaderNamesList.contains(headerName)) {
 						setHeaderConditional(part, headerName, getValue(headers, headerName, 0, null));
@@ -597,7 +624,7 @@ public class MimeMailBuilder {
 
 	}
 
-	private String getValue(JSONObject headers, String key, int i, String defaultVal) throws JSONException {
+    private String getValue(JSONObject headers, String key, int i, String defaultVal) throws JSONException {
 		if (headers == null || !headers.has(key))
 			return defaultVal;
 		Object value = headers.get(key);
@@ -621,6 +648,43 @@ public class MimeMailBuilder {
 		return defaultVal;
 	}
 
+    private void appendRecipientHeader(Part part, JSONObject headers, javax.mail.Message.RecipientType recipientType, String key) throws USMJSONAPIException {
+        if (headers == null || !headers.has(key))
+            return;
+        try {
+            if (!(part instanceof MimeMessage)) {
+                setHeaderConditional(part, key, getAddress(headers, key, null));
+                return;
+            }
+            MimeMessage message = (MimeMessage) part;
+            Object value = headers.get(key);
+            if (value instanceof String) {
+                message.setRecipients(recipientType, (String) value);
+            } else if (value instanceof JSONObject) {
+                message.addRecipient(recipientType, extractInternetAddress((JSONObject) value));
+            } else if (value instanceof JSONArray) {
+                JSONArray array = (JSONArray) value;
+                int length = array.length();
+                for (int i = 0; i < length; i++)
+                    message.addRecipient(recipientType, extractInternetAddress(array.getJSONObject(i)));
+            } else {
+                throw new IllegalArgumentException("header of unknown type" + value);
+            }
+        } catch (UnsupportedEncodingException e) {
+            throw new USMJSONAPIException(
+                ConnectorBundleErrorCodes.MIMEMAIL_UNSUPPORTED_ENCODING_ERROR_4,
+                ResponseStatusCode.INTERNAL_ERROR,
+                "unsupported encoding of text content: " + e.getMessage(),
+                e);
+        } catch (Exception e) {
+            throw new USMJSONAPIException(
+                ConnectorBundleErrorCodes.MIMEMAIL_RECIPIENT_EXCEPTION,
+                ResponseStatusCode.INTERNAL_ERROR,
+                "illegal mail address: " + e.getMessage(),
+                e);
+        }
+    }
+
 	private String getAddress(JSONObject headers, String key, String defaultVal) throws JSONException,
 			USMJSONAPIException {
 		if (headers == null || !headers.has(key))
@@ -650,20 +714,26 @@ public class MimeMailBuilder {
 			return defaultVal;
 		} catch (UnsupportedEncodingException e) {
 			throw new USMJSONAPIException(ConnectorBundleErrorCodes.MIMEMAIL_UNSUPPORTED_ENCODING_ERROR_2,
-					ResponseStatusCode.INTERNAL_ERROR, "unsupported encoding of text content: " + e.getMessage());
+					ResponseStatusCode.INTERNAL_ERROR, "unsupported encoding of text content: " + e.getMessage(), e);
 		} catch (Exception e) {
 			throw new USMJSONAPIException(ConnectorBundleErrorCodes.MIMEMAIL_ADDRESS_EXCEPTION,
-					ResponseStatusCode.INTERNAL_ERROR, "illegal mail address: " + e.getMessage());
+					ResponseStatusCode.INTERNAL_ERROR, "illegal mail address: " + e.getMessage(), e);
 		}
 		return defaultVal;
 	}
 
+    private QuotedInternetAddress extractInternetAddress(JSONObject address) throws JSONException, UnsupportedEncodingException, AddressException {
+        String addressPart = address.getString(ADDRESS);
+        String personalPart = address.optString(PERSONAL, null);
+        return new QuotedInternetAddress(addressPart, personalPart, UTF_8);
+    }
+
 	private String extractAddress(JSONObject address) throws JSONException, UnsupportedEncodingException {
-		String addressPart = address.getString(ADDRESS);
-		String personalPart = address.optString(PERSONAL, null);
 		try {
-			return new QuotedInternetAddress(addressPart, personalPart, UTF_8).toString();
+		    return extractInternetAddress(address).toString();
 		} catch (AddressException e) {
+	        String addressPart = address.getString(ADDRESS);
+	        String personalPart = address.optString(PERSONAL, null);
 			// In case of invalid address data, use simple composition of the raw (illegal) data
 			if (personalPart == null)
 				return addressPart;
