@@ -82,6 +82,7 @@ import org.apache.commons.httpclient.methods.multipart.MultipartRequestEntity;
 import org.apache.commons.httpclient.methods.multipart.Part;
 import org.apache.commons.httpclient.methods.multipart.StringPart;
 import org.apache.commons.httpclient.params.HttpConnectionManagerParams;
+import org.apache.commons.httpclient.params.HttpMethodParams;
 import org.apache.commons.logging.Log;
 import org.json.JSONArray;
 import org.json.JSONException;
@@ -106,6 +107,8 @@ import com.openexchange.usm.ox_json.OXResource;
  */
 public class OXJSONAccessImpl implements OXJSONAccess {
 
+    private static final String OX_REQUEST_INFORMATION_FIELD = "OXJSONAccessImpl.OXRequestInformation";
+
     private static final int MAX_RETRY = 10;
 
     private static final int MAX_REDIRECT_COUNT = 10;
@@ -147,7 +150,9 @@ public class OXJSONAccessImpl implements OXJSONAccess {
     // Check after 5 minutes if redirect still active, restricts number of log messages and redirect tests
     private static final long REDIRECT_RETEST_INTERVAL = 300000L;
 
-    private static final long LOCK_TIMEOUT = 30;
+    private static final long LOCK_TIMEOUT = 100;
+
+    private static final String USM_USER_AGENT = "Open-Xchange USM HTTP Client";
 
     private Log _journal;
 
@@ -213,6 +218,8 @@ public class OXJSONAccessImpl implements OXJSONAccess {
                 return new JSONObject().put(DATA, object);
             else if (object instanceof JSONObject)
                 return (JSONObject) object;
+            if (object instanceof JSONArray)
+                return new JSONObject().put(DATA, object);
             else if (object instanceof Integer)
                 return new JSONObject().put(DATA, String.valueOf(object));
             else
@@ -234,6 +241,7 @@ public class OXJSONAccessImpl implements OXJSONAccess {
         String password = session.getPassword();
         HttpClient client = new HttpClient(_httpManager);
         client.getParams().setParameter("http.protocol.content-charset", "UTF-8");
+        client.getParams().setParameter(HttpMethodParams.USER_AGENT, USM_USER_AGENT);
         PostMethod method = new PostMethod(
             getBaseURL() + "login?client=" + urlEncode("USM-" + session.getProtocol()) + "&version=" + urlEncode(USMVersion.VERSION_BUILD) + "&authId=" + UUID.randomUUID() + createClientIPParameter(session));
         method.addParameter("action", "login");
@@ -277,7 +285,7 @@ public class OXJSONAccessImpl implements OXJSONAccess {
     private void logout(Session session, boolean doLog) {
         if (doLog && _journal.isDebugEnabled())
             _journal.debug(session + " Logging out");
-        if (!obtainOptionalLock(session)) {
+        if (!obtainOptionalLock(session, "login?action=logout")) {
             _journal.warn("Timeout while waiting to log out session " + session);
             session.setOXConnectionData(null);
             return;
@@ -434,7 +442,7 @@ public class OXJSONAccessImpl implements OXJSONAccess {
     }
 
     private OXConnectionData obtainConnectionData(Session session) throws AuthenticationFailedException, OXCommunicationException {
-        obtainRequiredLock(session);
+        obtainRequiredLock(session, "<obtain OX connection data>");
         try {
             OXConnectionInformation o = session.getOXConnectionData();
             _journal.debug(session + " Retrieving OX session with ID: " + o);
@@ -473,7 +481,7 @@ public class OXJSONAccessImpl implements OXJSONAccess {
     }
 
     private JSONResult executeMethodAndExtractResult(Session session, OXConnectionData data, HttpMethodBase method, String action, Map<String, String> parameters) throws AuthenticationFailedException, OXCommunicationException {
-        obtainRequiredLock(session);
+        obtainRequiredLock(session, method.getName() + ' ' + method.getPath() + '?' + method.getQueryString());
         try {
             JSONResult result = executeMethodOnce(session, data.getHttpClient(), method);
             // On specific error for expired/unknown session, do login and repeat
@@ -483,6 +491,7 @@ public class OXJSONAccessImpl implements OXJSONAccess {
                 login(session);
                 data = obtainConnectionData(session);
                 method.setQueryString(generateQueryParameters(action, data.getSessionID(), parameters));
+                setRequestInformation(session, method.getName() + ' ' + method.getPath() + '?' + method.getQueryString());
                 result = executeMethodOnce(session, data.getHttpClient(), method);
             }
             for (int i = 0; i < MAX_RETRY && shouldRetryRequest(result); i++) {
@@ -773,28 +782,40 @@ public class OXJSONAccessImpl implements OXJSONAccess {
         return result;
     }
 
-    private boolean obtainOptionalLock(Session session) {
+    private boolean obtainOptionalLock(Session session, String requestInformation) {
         try {
-            return session.getOXConnectionLock().tryLock(LOCK_TIMEOUT, TimeUnit.SECONDS);
+            return tryLock(session, requestInformation);
         } catch (InterruptedException e) {
             return false;
         }
     }
 
-    private void obtainRequiredLock(Session session) throws OXCommunicationException {
+    private void obtainRequiredLock(Session session, String requestInformation) throws OXCommunicationException {
         try {
-            if (session.getOXConnectionLock().tryLock(LOCK_TIMEOUT, TimeUnit.SECONDS))
+            if (tryLock(session, requestInformation))
                 return;
         } catch (InterruptedException e) {
             // fall through
         }
         throw new OXCommunicationException(
             OXJSONErrorCode.FAILED_TO_GAIN_EXCLUSIVE_OX_ACCESS,
-            "Timeout while waiting for exclusive access to OX server",
-            "timeout");
+            "Timeout while waiting for exclusive access to OX server, locked by " + session + '/' + session.getCustomProperty(OX_REQUEST_INFORMATION_FIELD),
+            (String) null);
+    }
+
+    private boolean tryLock(Session session, String requestInformation) throws InterruptedException {
+        boolean success = session.getOXConnectionLock().tryLock(LOCK_TIMEOUT, TimeUnit.SECONDS);
+        if (success)
+            setRequestInformation(session, requestInformation);
+        return success;
     }
 
     private void releaseLock(Session session) {
+        setRequestInformation(session, null);
         session.getOXConnectionLock().unlock();
     }
+
+    private void setRequestInformation(Session session, String requestInformation) {
+        session.setCustomProperty(OX_REQUEST_INFORMATION_FIELD, requestInformation);
+    }
 }
