@@ -95,6 +95,8 @@ public class SimpleDataObject implements DataObject {
 
 	protected final Map<String, Integer> _fieldNameToIndexMap;
 
+	protected final int _lastModifiedIndex;
+	
 	protected ChangeState _changeState = ChangeState.CREATED;
 
 	protected final Object[] _originalFieldContent;
@@ -131,6 +133,8 @@ public class SimpleDataObject implements DataObject {
 			_fieldContent[i] = fieldType.createNewObject();
 		}
 		_fieldNameToIndexMap = FieldMapStorageImpl.getInstance().getFieldNameToIndexMap(contentType);
+		Integer lastModifiedIndex = _fieldNameToIndexMap.get("last_modified");
+		_lastModifiedIndex = (lastModifiedIndex != null) ? lastModifiedIndex.intValue() : -1;
 	}
 
 	protected SimpleDataObject(SimpleDataObject source, boolean linkUUIDs) {
@@ -140,6 +144,7 @@ public class SimpleDataObject implements DataObject {
 		_session = source._session;
 		_contentType = source._contentType;
 		_fieldNameToIndexMap = source._fieldNameToIndexMap;
+		_lastModifiedIndex = source._lastModifiedIndex;
 		_changeState = source._changeState;
 		_timestamp = source._timestamp;
 		_originalTimestamp = source._originalTimestamp;
@@ -415,7 +420,7 @@ public class SimpleDataObject implements DataObject {
 		if (getClass() != obj.getClass())
 			return false;
 		SimpleDataObject other = (SimpleDataObject) obj;
-		return internalEquals(other, false);
+		return internalEquals(other, false, true);
 	}
 
 	/**
@@ -425,32 +430,36 @@ public class SimpleDataObject implements DataObject {
 	public boolean equalsWithoutID(DataObject other) {
 		if (!(other instanceof SimpleDataObject))
 			return false;
-		return internalEquals((SimpleDataObject) other, true);
+		return internalEquals((SimpleDataObject) other, true, true);
+	}
+	
+	public boolean equalsWithLastModified(DataObject other) {
+        if (!(other instanceof SimpleDataObject))
+            return false;
+        return internalEquals((SimpleDataObject) other, false, false);
 	}
 
-	private boolean internalEquals(SimpleDataObject other, boolean ignoreID) {
-		if (!_changeState.equals(other._changeState))
-			return false;
-		if (!_contentType.equals(other._contentType))
-			return false;
-		if (!_session.equals(other._session))
-			return false;
-		//		if (_timestamp != other._timestamp)
-		//			return false;
-		//		if (_originalTimestamp != other._originalTimestamp)
-		//			return false;
-		ContentTypeField[] fields = _contentType.getFields();
-		for (int i = 0; i < _fieldContent.length; i++) {
-			if (!ignoreID || i != ID_INDEX) {
-				DataType<?> type = fields[i].getFieldType();
-				if (!type.isEqual(_fieldContent[i], other._fieldContent[i]))
-					return false;
-				if (!type.isEqual(_originalFieldContent[i], other._originalFieldContent[i]))
-					return false;
-			}
-		}
-		return true;
-	}
+    private boolean internalEquals(SimpleDataObject other, boolean ignoreID, boolean ignoreLastModified) {
+        if (!_changeState.equals(other._changeState))
+            return false;
+        if (!_contentType.equals(other._contentType))
+            return false;
+        if (!_session.equals(other._session))
+            return false;
+        ContentTypeField[] fields = _contentType.getFields();
+        for (int i = 0; i < _fieldContent.length; i++) {
+            if (ignoreLastModified && i == _lastModifiedIndex)
+                continue;
+            if (ignoreID && i == ID_INDEX)
+                continue;
+            DataType<?> type = fields[i].getFieldType();
+            if (!type.isEqual(_fieldContent[i], other._fieldContent[i]))
+                return false;
+            if (!type.isEqual(_originalFieldContent[i], other._originalFieldContent[i]))
+                return false;
+        }
+        return true;
+    }
 
 	public DataObject createCopy(boolean linkUUIDs) {
 		return new SimpleDataObject(this, linkUUIDs);
