@@ -340,7 +340,7 @@ public class SessionImpl implements Session, Serializable {
         try {
             _syncedFolderContent.put(folderId, 0L);
             return _sessionManager.getIncrementalSyncer().syncChangesWithServer(
-                createFolderElementsStorage(folder, filter, storeResult, timestamp),
+                createFolderElementsStorage(folder, filter, storeResult, timestamp, resultLimit),
                 getConflictResolution(conflictResolution),
                 getCachedFolderElements(folderId, contentType, timestamp),
                 elements,
@@ -368,7 +368,7 @@ public class SessionImpl implements Session, Serializable {
         try {
             _syncedFolderContent.put(folderId, 0L);
             return _sessionManager.getSlowSyncer().syncWithServer(
-                createFolderElementsStorage(folder, filter, storeResult, 0L),
+                createFolderElementsStorage(folder, filter, storeResult, 0L, limit),
                 getConflictResolution(conflictResolution),
                 limit,
                 filter,
@@ -384,15 +384,9 @@ public class SessionImpl implements Session, Serializable {
         }
     }
 
-    private FolderElementsStorage createFolderElementsStorage(Folder folder, DataObjectFilter filter, boolean storeResult, long oldTimestamp) {
+    private FolderElementsStorage createFolderElementsStorage(Folder folder, DataObjectFilter filter, boolean storeResult, long oldTimestamp, int limit) {
         ContentType contentType = folder.getElementsContentType();
-        return new FolderElementsStorage(
-            this,
-            folder,
-            getValidFilter(contentType),
-            (filter != null) ? filter : _syncServerObjectFilter,
-            storeResult,
-            oldTimestamp);
+        return new FolderElementsStorage(this, folder, (filter != null) ? filter : _syncServerObjectFilter, storeResult, oldTimestamp, limit);
     }
 
     public BitSet getValidFilter(ContentType elementsContentType) {
