@@ -58,7 +58,6 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.UUID;
-import java.util.concurrent.atomic.AtomicInteger;
 import javax.management.MalformedObjectNameException;
 import javax.management.NotCompliantMBeanException;
 import javax.management.ObjectName;
@@ -432,20 +431,18 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
         SessionID id = new SessionID(user, protocol, device);
         SessionImpl session = _sessionStorage.getSession(id);
         if (session != null) {
-            if (session.checkProperDBStorage()) {
+            try {
                 if (!session.getPassword().equals(password)) {
                     session.setPassword(password);
                     session.setClientIp(clientIP);
                     _oxAjaxAccess.login(session);
                 }
-                try {
-                    session.updateClientIp(clientIP);
-                    recheckUSMAccess(initializer, session);
+                session.updateClientIp(clientIP);
+                if (recheckUSMAccess(initializer, session))
                     return session;
-                } catch (USMAccessDeniedException e) {
-                    _sessionStorage.removeSession(session);
-                    throw e;
-                }
+            } catch (USMException e) {
+                _sessionStorage.removeSession(session);
+                throw e;
             }
             // Cached Session is no longer present in DB, start from scratch
             _sessionStorage.removeSession(session);
@@ -464,13 +461,14 @@ public class SessionManagerImpl implements SessionManager, OXEventListener {
         return session;
     }
 
-    private void recheckUSMAccess(SessionInitializer initializer, SessionImpl session) throws USMException {
+    private boolean recheckUSMAccess(SessionInitializer initializer, SessionImpl session) throws USMException {
         long now = System.currentTimeMillis();
-        if (session.getLastAccessCheck() < now - ACCESS_CHECK_INTERVAL) {
-            checkUSMAccess(session, null);
-            initializer.checkAccess(session);
-            session.setLastAccessCheck(now);
-        }
+        if (session.getLastAccessCheck() >= now - ACCESS_CHECK_INTERVAL)
+            return true;
+        checkUSMAccess(session, null);
+        initializer.checkAccess(session);
+        session.setLastAccessCheck(now);
+        return session.checkProperDBStorage();
     }
 
     private void checkUSMAccess(SessionImpl session, JSONObject configuration) throws USMAccessDeniedException, AuthenticationFailedException, OXCommunicationException {
