@@ -49,8 +49,8 @@
 package com.openexchange.usm.session.sync;
 
 import java.io.Serializable;
-import com.openexchange.usm.api.session.Session;
 import com.openexchange.usm.session.impl.SessionID;
+import com.openexchange.usm.session.impl.SessionImpl;
 
 /**
  * 
@@ -65,7 +65,7 @@ public class OXDataCacheID implements Serializable {
 	 * Constructor used to identify folder hierarchy
 	 * @param session
 	 */
-	public OXDataCacheID(Session session) {
+	public OXDataCacheID(SessionImpl session) {
 		this(session, null);
 	}
 
@@ -75,8 +75,8 @@ public class OXDataCacheID implements Serializable {
 	 * @param session
 	 * @param folderID
 	 */
-	public OXDataCacheID(Session session, String folderID) {
-		_sessionID = session == null ? null : new SessionID(session);
+	public OXDataCacheID(SessionImpl session, String folderID) {
+		_sessionID = session == null ? null : session.getCompleteSessionID().getSessionID();
 		_folderID = folderID;
 	}
 
