@@ -48,12 +48,18 @@
  */
 package com.openexchange.usm.util;
 
-import java.io.*;
-import java.sql.*;
+import java.io.Closeable;
+import java.io.IOException;
+import java.io.UnsupportedEncodingException;
+import java.sql.Connection;
+import java.sql.ResultSet;
+import java.sql.SQLException;
+import java.sql.Statement;
 import java.util.ArrayList;
-
+import java.util.BitSet;
 import org.apache.commons.codec.binary.Base64;
-
+import com.openexchange.usm.api.contenttypes.ContentType;
+import com.openexchange.usm.api.contenttypes.ContentTypeField;
 import com.openexchange.usm.api.database.DatabaseAccessException;
 import com.openexchange.usm.api.database.EncapsulatedConnection;
 import com.openexchange.usm.api.exceptions.USMIllegalStateException;
@@ -226,4 +232,26 @@ public class Toolkit {
 		return new ArrayList<T>(initialCapacity);
 	}
 
+    /**
+     * @param array Array of objects
+     * @param object Object to search for
+     * @return true if at least 1 (non-null) element of the array is equal to the given object
+     */
+    public static boolean contains(Object[] array, Object object) {
+        for (Object o : array) {
+            if (o != null && o.equals(object))
+                return true;
+        }
+        return false;
+    }
+
+    public static BitSet buildFieldBitSet(ContentType contentType, String... fields) {
+        BitSet set = new BitSet();
+        ContentTypeField[] contentTypeFields = contentType.getFields();
+        for (int i = 0; i < contentTypeFields.length; i++) {
+            if (contains(fields, contentTypeFields[i].getFieldName()))
+                set.set(i);
+        }
+        return set;
+    }
 }
